@@ -1,31 +1,32 @@
-from ast import literal_eval as ast_literal_eval
 import re
 
+from ast  import literal_eval as ast_literal_eval
+from copy import deepcopy
+
 import cfdm
 
 from .functions import inspect as cf_inspect
 
 from .data.data import Data
 
-from .functions import (_DEPRECATION_ERROR_KWARGS,
-                        _DEPRECATION_ERROR_METHOD)
+from .functions import _DEPRECATION_ERROR_METHOD
 
 from .decorators import (_inplace_enabled,
                          _inplace_enabled_define_and_cleanup,
-                         _deprecation_error_i_kwarg)
+                         _deprecated_kwarg_check)
 
 
 _collapse_cell_methods = {
-    'max'            : 'maximum',
-    'mean'           : 'mean',
-    'mid_range'      : 'mid_range',
-    'min'            : 'minimum',
-    'range'          : 'range',
-    'sd'             : 'standard_deviation',
-    'sum'            : 'sum',
-    'var'            : 'variance',
-    'sample_size'    : None, 
-    'sum_of_weights' : None,
+    'max': 'maximum',
+    'mean': 'mean',
+    'mid_range': 'mid_range',
+    'min': 'minimum',
+    'range': 'range',
+    'sd': 'standard_deviation',
+    'sum': 'sum',
+    'var': 'variance',
+    'sample_size': None,
+    'sum_of_weights': None,
     'sum_of_weights2': None,
     }
 
@@ -37,7 +38,7 @@ class CellMethod(cfdm.CellMethod):
     the field construct represent the variation of the physical
     quantity within its cells, i.e. the structure of the data at a
     higher resolution.
-    
+
     A single cell method construct consists of a set of axes, a
     "method" property which describes how a value of the field
     construct's data array describes the variation of the quantity
@@ -56,22 +57,21 @@ class CellMethod(cfdm.CellMethod):
         '''
         return super().__repr__().replace('<', '<CF ', 1)
 
-    
     @classmethod
     def create(cls, cell_methods_string=None):
-        '''Parse a CF-like cell_methods string.   
-    
+        '''Parse a CF-like cell_methods string.
+
     :Parameters:
-    
+
         cell_methods_string: `str`
             A CF cell_methods string.
-    
+
     :Returns:
-    
+
         `list`
-    
+
     **Examples:**
-    
+
     TODO
 
         '''
@@ -81,24 +81,24 @@ class CellMethod(cfdm.CellMethod):
 
         if not cell_methods_string:
             return out
-        
+
         # ------------------------------------------------------------
         # Split the cell_methods string into a list of strings ready
         # for parsing. For example:
         #
         #   'lat: mean (interval: 1 hour)'
-        # 
+        #
         # would be split up into:
         #
         #   ['lat:', 'mean', '(', 'interval:', '1', 'hour', ')']
         # ------------------------------------------------------------
-        cell_methods = re.sub('\((?=[^\s])' , '( ', cell_methods_string)
+        cell_methods = re.sub('\((?=[^\s])', '( ', cell_methods_string)
         cell_methods = re.sub('(?<=[^\s])\)', ' )', cell_methods).split()
 
         while cell_methods:
             cm = cls()
 
-            axes  = []
+            axes = []
             while cell_methods:
                 if not cell_methods[0].endswith(':'):
                     break
@@ -110,7 +110,7 @@ class CellMethod(cfdm.CellMethod):
                 axis = cell_methods.pop(0)[:-1]
 
                 axes.append(axis)
-            #--- End: while
+            # --- End: while
             cm.set_axes(axes)
 
             if not cell_methods:
@@ -119,7 +119,7 @@ class CellMethod(cfdm.CellMethod):
 
             # Method
             cm.set_method(cell_methods.pop(0))
-            
+
             if not cell_methods:
                 out.append(cm)
                 break
@@ -131,8 +131,8 @@ class CellMethod(cfdm.CellMethod):
                 cm.set_qualifier(attr, cell_methods.pop(0))
                 if not cell_methods:
                     break
-            #--- End: while
-            if not cell_methods: 
+            # --- End: while
+            if not cell_methods:
                 out.append(cm)
                 break
 
@@ -143,7 +143,7 @@ class CellMethod(cfdm.CellMethod):
 
                 if not (re.search('^(interval|comment):$', cell_methods[0])):
                     cell_methods.insert(0, 'comment:')
-                           
+
                 while not re.search('^\)$', cell_methods[0]):
                     term = cell_methods.pop(0)[:-1]
 
@@ -157,16 +157,21 @@ class CellMethod(cfdm.CellMethod):
                         try:
                             parsed_interval = ast_literal_eval(interval)
                         except (SyntaxError, ValueError):
-                            raise ValueError("{}: {!r}".format(incorrect_interval, interval))
+                            raise ValueError(
+                                "{}: {!r}".format(incorrect_interval, interval)
+                            )
 
                         try:
-                            data = Data(array=parsed_interval, units=units, copy=False)
+                            data = Data(
+                                array=parsed_interval, units=units, copy=False)
                         except:
-                            raise ValueError("{}: {!r}".format(incorrect_interval, interval))
+                            raise ValueError(
+                                "{}: {!r}".format(incorrect_interval, interval)
+                            )
 
                         intervals.append(data)
                         continue
-                    #--- End: if
+                    # --- End: if
 
                     if term == 'comment':
                         comment = []
@@ -176,27 +181,28 @@ class CellMethod(cfdm.CellMethod):
                             if cell_methods[0].endswith(':'):
                                 break
                             comment.append(cell_methods.pop(0))
-                        #--- End: while
+                        # --- End: while
                         cm.set_qualifier('comment', ' '.join(comment))
-                #--- End: while 
+                # --- End: while
 
                 if cell_methods[0].endswith(')'):
                     cell_methods.pop(0)
-            #--- End: if
+            # --- End: if
 
-            n_intervals = len(intervals)          
+            n_intervals = len(intervals)
             if n_intervals > 1 and n_intervals != len(axes):
-                raise ValueError("{} (doesn't match axes): {!r}".format(incorrect_interval, interval))
+                raise ValueError(
+                    "{} (doesn't match axes): {!r}".format(
+                        incorrect_interval, interval))
 
             if intervals:
                 cm.set_qualifier('interval', intervals)
 
             out.append(cm)
-        #--- End: while
+        # --- End: while
 
         return out
 
-
     def __hash__(self):
         '''
 
@@ -205,31 +211,28 @@ class CellMethod(cfdm.CellMethod):
         '''
         return hash(str(self))
 
-
     def __eq__(self, y):
         '''x.__eq__(y) <==> x==y
 
         '''
         return self.equals(y)
 
-
     def __ne__(self, other):
         '''x.__ne__(y) <==> x!=y
 
         '''
         return not self.__eq__(other)
 
-
     @property
     def within(self):
         '''The cell method's within qualifier.
-    
+
     These describe how climatological statistics have been derived.
-    
+
     .. seealso:: `over`
-    
+
     **Examples:**
-    
+
     >>> c
     >>> c
     <CF CellMethod: time: minimum>
@@ -244,21 +247,23 @@ class CellMethod(cfdm.CellMethod):
 
         '''
         return self.get_qualifier('within', default=AttributeError())
+
     @within.setter
     def within(self, value): self.set_qualifier('within', value)
+
     @within.deleter
-    def within(self):        self.del_qualifier('within', default=AttributeError())
+    def within(self): self.del_qualifier('within', default=AttributeError())
 
     @property
     def where(self):
         '''The cell method's where qualifier.
 
     These describe how climatological statistics have been derived.
-    
+
     .. seealso:: `over`
-    
+
     **Examples:**
-    
+
     >>> c
     >>> c
     <CF CellMethod: time: minimum>
@@ -273,21 +278,23 @@ class CellMethod(cfdm.CellMethod):
 
         '''
         return self.get_qualifier('where', default=AttributeError())
+
     @where.setter
     def where(self, value): self.set_qualifier('where', value)
+
     @where.deleter
-    def where(self):        self.del_qualifier('where', default=AttributeError())
+    def where(self): self.del_qualifier('where', default=AttributeError())
 
     @property
     def over(self):
         '''The cell method's over qualifier.
 
     These describe how climatological statistics have been derived.
-    
+
     .. seealso:: `within`
-    
+
     **Examples:**
-    
+
     >>> c
     >>> c
     <CF CellMethod: time: minimum>
@@ -302,8 +309,10 @@ class CellMethod(cfdm.CellMethod):
 
         '''
         return self.get_qualifier('over', default=AttributeError())
+
     @over.setter
     def over(self, value): self.set_qualifier('over', value)
+
     @over.deleter
     def over(self):        self.del_qualifier('over', default=AttributeError())
 
@@ -313,19 +322,21 @@ class CellMethod(cfdm.CellMethod):
 
         '''
         return self.get_qualifier('comment', default=AttributeError())
+
     @comment.setter
     def comment(self, value): self.set_qualifier('comment', value)
+
     @comment.deleter
-    def comment(self):        self.del_qualifier('comment', default=AttributeError())
+    def comment(self): self.del_qualifier('comment', default=AttributeError())
 
     @property
     def method(self):
         '''The cell method's method qualifier.
 
     Describes how the cell values have been determined or derived.
-    
+
     **Examples:**
-    
+
     >>> c
     <CF CellMethod: time: minimum>
     >>> c.method
@@ -339,8 +350,10 @@ class CellMethod(cfdm.CellMethod):
 
         '''
         return self.get_method(default=AttributeError())
+
     @method.setter
     def method(self, value):  self.set_method(value)
+
     @method.deleter
     def method(self):
         self.del_method(default=AttributeError())
@@ -350,7 +363,7 @@ class CellMethod(cfdm.CellMethod):
         '''The cell method's interval qualifier(s).
 
     **Examples:**
-    
+
     >>> c
     <CF CellMethod: time: minimum>
     >>> c.intervals
@@ -365,10 +378,10 @@ class CellMethod(cfdm.CellMethod):
     <CF CellMethod: time: minimum (interval: 7.5 minutes)>
     >>> c.intervals
     (<CF Data: 7.5 minutes>,)
-    >>> del c.intervals        
+    >>> del c.intervals
     >>> c
     <CF CellMethods: time: minimum>
-    
+
     >>> c
     <CF CellMethod: lat: lon: mean>
     >>> c.intervals = ['0.2 degree_N', cf.Data(0.1 'degree_E')]
@@ -381,10 +394,9 @@ class CellMethod(cfdm.CellMethod):
     @intervals.setter
     def intervals(self, value):
         if not isinstance(value, (tuple, list)):
-            raise ValueError(
-                "intervals attribute must be a tuple or list, not {0!r}".format(
-                    value.__class__.__name__))
-        
+            msg = "intervals attribute must be a tuple or list, not {0!r}"
+            raise ValueError(msg.format(value.__class__.__name__))
+
         # Parse the intervals
         values = []
         for interval in value:
@@ -401,7 +413,7 @@ class CellMethod(cfdm.CellMethod):
                     units = ' '.join(i)
                 else:
                     units = None
-                    
+
                 try:
                     d = Data(x, units)
                 except:
@@ -413,17 +425,17 @@ class CellMethod(cfdm.CellMethod):
                 except:
                     raise ValueError(
                         "Unparseable interval: {0!r}".format(interval))
-            #--- End: if
-            
+            # --- End: if
+
             if d.size != 1:
                 raise ValueError(
                     "Unparseable interval: {0!r}".format(interval))
-                
+
             if d.ndim > 1:
                 d.squeeze(inplace=True)
 
             values.append(d)
-        #--- End: for
+        # --- End: for
 
         self.set_qualifier('interval', tuple(values))
 
@@ -431,30 +443,129 @@ class CellMethod(cfdm.CellMethod):
     def intervals(self):
         self.del_qualifier('interval', default=AttributeError())
 
-        
     @property
     def axes(self):
         '''TODO
         '''
         return self.get_axes(default=AttributeError())
+
     @axes.setter
     def axes(self, value):
         if not isinstance(value, (tuple, list)):
             raise ValueError(
                 "axes attribute must be a tuple or list, not {0}".format(
                     value.__class__.__name__))
-        
+
         self.set_axes(tuple(value))
+
     @axes.deleter
     def axes(self):
         self.del_axes(default=AttributeError())
 
+    # ----------------------------------------------------------------
+    # Methods
+    # ----------------------------------------------------------------
+    def creation_commands(self, representative_data=False,
+                          namespace='cf', indent=0, string=True,
+                          name='c'):
+        '''Return the commands that would create the cell measure construct.
 
-    @_deprecation_error_i_kwarg
+    .. versionadded:: 3.2.0
+
+    .. seealso:: `cf.Data.creation_commands`,
+                 `cf.Field.creation_commands`
+
+    :Parameters:
+
+        representative_data: `bool`, optional
+            Ignored.
+
+        namespace: `str`, optional
+            The namespace containing classes of the ``cf``
+            package. This is prefixed to the class name in commands
+            that instantiate instances of ``cf`` objects. By default,
+            *namespace* is ``'cf'``, i.e. it is assumed that ``cf``
+            was imported as ``import cf``.
+
+            *Parameter example:*
+              If ``cf`` was imported as ``import cf as cfp`` then set
+              ``namespace='cfp'``
+
+            *Parameter example:*
+              If ``cf`` was imported as ``from cf import *`` then set
+              ``namespace=''``
+
+        indent: `int`, optional
+            Indent each line by this many spaces. By default no
+            indentation is applied. Ignored if *string* is False.
+
+        string: `bool`, optional
+            If False then return each command as an element of a
+            `list`. By default the commands are concatenated into
+            a string, with a new line inserted between each command.
+
+    :Returns:
+
+        `str` or `list`
+            The commands in a string, with a new line inserted between
+            each command. If *string* is False then the separate
+            commands are returned as each element of a `list`.
+
+    **Examples:**
+
+        TODO
+        '''
+        namespace0 = namespace
+        if namespace0:
+            namespace = namespace+"."
+        else:
+            namespace = ""
+
+        indent = ' ' * indent
+
+        out = []
+        out.append("# {}".format(self.construct_type))
+        out.append("{} = {}{}()".format(name, namespace,
+                                        self.__class__.__name__))
+        method = self.get_method(None)
+        if method is not None:
+            out.append("{}.set_method({!r})".format(name, method))
+
+        axes = self.get_axes(None)
+        if axes is not None:
+            out.append("{}.set_axes({!r})".format(name, axes))
+            
+        for term, value in self.qualifiers().items():
+            if term == 'interval':
+                value = deepcopy(value)
+                for i, data in enumerate(value[:]):
+                    if isinstance(data, Data):
+                        value[i] = data.creation_commands(
+                            name=None, namespace=namespace0,
+                            indent=0, string=True)
+                    else:
+                        value[i] = str(data)
+                # --- End: for
+                
+                value = ', '.join(value)
+                value = "["+value+"]"
+            else:
+                value = repr(value)
+
+            out.append("{}.set_qualifier({!r}, {})".format(name, term,
+                                                           value))
+
+        if string:
+            out[0] = indent+out[0]
+            out = ('\n'+indent).join(out)
+
+        return out
+
+    @_deprecated_kwarg_check('i')
     @_inplace_enabled
     def expand_intervals(self, inplace=False, i=False):
         '''TODO
-        
+
         '''
         c = _inplace_enabled_define_and_cleanup(self)
         n_axes = len(c.get_axes(()))
@@ -464,16 +575,15 @@ class CellMethod(cfdm.CellMethod):
 
         return c
 
-
-    @_deprecation_error_i_kwarg
+    @_deprecated_kwarg_check('i')
     @_inplace_enabled
     def change_axes(self, axis_map, inplace=False, i=False):
         '''TODO
 
     :Parameters:
-    
+
         axis_map: `dict`
-    
+
         inplace: `bool`
 
         '''
@@ -486,40 +596,37 @@ class CellMethod(cfdm.CellMethod):
 
         return c
 
-
+    @_deprecated_kwarg_check('traceback')
     def equivalent(self, other, rtol=None, atol=None, verbose=False,
                    traceback=False):
         '''True if two cell methods are equivalent, False otherwise.
 
     The `axes` and `intervals` attributes are ignored in the
     comparison.
-    
+
     :Parameters:
-    
-        other: 
+
+        other:
             The object to compare for equality.
-    
+
         atol: `float`, optional
             The absolute tolerance for all numerical comparisons, By
             default the value returned by the `ATOL` function is used.
-    
+
         rtol: `float`, optional
             The relative tolerance for all numerical comparisons, By
             default the value returned by the `RTOL` function is used.
-    
-    :Returns: 
-    
+
+    :Returns:
+
         `bool`
             Whether or not the two instances are equivalent.
-    
+
     **Examples:**
-    
+
     TODO
 
         '''
-        if traceback:
-            _DEPRECATION_ERROR_KWARGS(self, 'equivalent', traceback=True) # pragma: no cover
-            
         if self is other:
             return True
 
@@ -527,18 +634,21 @@ class CellMethod(cfdm.CellMethod):
         if self.__class__ != other.__class__:
             if verbose:
                 print("{0}: Different types: {0} != {1}".format(
-                    self.__class__.__name__, other.__class__.__name__)) # pragma: no cover
+                    self.__class__.__name__,
+                    other.__class__.__name__
+                    )
+                )  # pragma: no cover
             return False
 
         axes0 = self.get_axes(())
         axes1 = other.get_axes(())
-            
+
         if len(axes0) != len(axes1) or set(axes0) != set(axes1):
             if verbose:
                 print("{}: Nonequivalent axes: {!r}, {!r}".format(
-                    self.__class__.__name__, axes0, axes1)) # pragma: no cover
+                    self.__class__.__name__, axes0, axes1))  # pragma: no cover
             return False
-            
+
 #        other1 = other.copy()
         argsort = [axes1.index(axis0) for axis0 in axes0]
         other1 = other.sorted(indices=argsort)
@@ -547,51 +657,58 @@ class CellMethod(cfdm.CellMethod):
                            ignore_qualifiers=('interval',)):
             if verbose:
                 print("{0}: Nonequivalent: {1!r}, {2!r}".format(
-                    self.__class__.__name__, self, other)) # pragma: no cover
+                    self.__class__.__name__, self, other))  # pragma: no cover
             return False
 
         self1 = self
-        if len(self1.get_qualifier('interval', ())) != len(other1.get_qualifier('interval', ())):
+        if len(self1.get_qualifier('interval', ())) != len(
+                other1.get_qualifier('interval', ())):
             self1 = self1.expand_intervals()
             other1.expand_intervals(inplace=True)
-            if len(self1.get_qualifier('interval', ())) != len(other1.get_qualifier('interval', ())):
+            if len(self1.get_qualifier('interval', ())) != len(
+                    other1.get_qualifier('interval', ())):
                 if verbose:
-                    print("{0}: Different numbers of intervals: {1!r} != {2!r}".format(
+                    msg = "{0}: Different numbers of intervals: {1!r} != {2!r}"
+                    print(msg.format(
                         self.__class__.__name__,
                         self1.get_qualifier('interval', ()),
-                        other1.get_qualifier('interval', ()))) # pragma: no cover
+                        other1.get_qualifier('interval', ())
+                        )
+                    )  # pragma: no cover
                 return False
 
-        intervals0 = self1.get_qualifier('interval', ())                   
+        intervals0 = self1.get_qualifier('interval', ())
         if intervals0:
-            for data0, data1 in zip(intervals0, other1.get_qualifier('interval', ())):
+            for data0, data1 in zip(
+                    intervals0, other1.get_qualifier('interval', ())):
                 if not data0.allclose(data1, rtol=rtol, atol=atol):
                     if verbose:
-                        print("{0}: Different interval data: {1!r} != {2!r}".format(
-                            self.__class__.__name__, self.intervals, other.intervals)) # pragma: no cover
+                        msg = "{0}: Different interval data: {1!r} != {2!r}"
+                        print(msg.format(
+                            self.__class__.__name__,
+                            self.intervals, other.intervals)
+                        )  # pragma: no cover
                     return False
-        #--- End: if
+        # --- End: if
 
         # Still here? Then they are equivalent
         return True
 
-
     def inspect(self):
         '''Inspect the attributes.
 
     .. seealso:: `cf.inspect`
-    
-    :Returns: 
-    
+
+    :Returns:
+
         `None`
 
         '''
         print(cf_inspect(self))
 
-
-    # ----------------------------------------------------------------    
+    # ----------------------------------------------------------------
     # Deprecated attributes and methods
-    # ----------------------------------------------------------------    
+    # ----------------------------------------------------------------
     def write(self, axis_map=None):
         '''Return a string of the cell method.
 
@@ -602,15 +719,16 @@ class CellMethod(cfdm.CellMethod):
         if axis_map is None:  # distinguish from falsy '{}'
             axis_map = {}
         _DEPRECATED_ERROR_METHOD(
-            self, 'write', "Use 'str(cell_method)' instead.") # Pragma: no cover
-
+            self, 'write', "Use 'str(cell_method)' instead."
+        )  # pragma: no cover
 
     def remove_axes(self, axes):
         '''Deprecated at version 3.0.0. Use method 'del_axes' instead."
 
         '''
         _DEPRECATION_ERROR_METHOD(
-            self, 'remove_axes', "Use method 'del_axes' instead.") # pragma: no cover
+            self, 'remove_axes', "Use method 'del_axes' instead."
+        )  # pragma: no cover
 
-        
-#--- End: class
+
+# --- End: class
