@@ -7,7 +7,7 @@ from numpy import empty       as numpy_empty
 from numpy import expand_dims as numpy_expand_dims
 from numpy import squeeze     as numpy_squeeze
 
-from copy      import deepcopy
+from copy import deepcopy
 
 from operator import mul
 
@@ -22,6 +22,7 @@ from ..decorators import (_inplace_enabled,
 
 _empty_matrix = numpy_empty((), dtype=object)
 
+
 class PartitionMatrix:
     '''
 
@@ -41,8 +42,8 @@ size 1.
 ==========  ===========================================================
 Attribute   Description
 ==========  ===========================================================
-`!axes`     
-`!matrix`     
+`!axes`
+`!matrix`
 `!ndim`     The number of partition dimensions in the partition matrix.
 `!shape`    List of the partition matrix's dimension sizes.
 `!size`     The number of partitions in the partition matrix.
@@ -52,41 +53,41 @@ Attribute   Description
 
     def __init__(self, matrix, axes):
         '''**Initialization**
-    
+
     :Parameters:
-    
+
         matrix: `numpy.ndarray`
             An array of Partition objects.
-    
+
         axes: `list`
             The identities of the partition axes of the partition
             array. If the partition matrix is a scalar array then it
             is an empty list. DO NOT UPDATE INPLACE.
-    
+
     **Examples:**
-    
+
     >>> pm = PartitionMatrix(
-    ...          numpy.array(Partition(location    = [(0, 1), (2, 4)],
-    ...                                shape       = [1, 2],
-    ...                                _dimensions = ['dim2', 'dim0'],
-    ...                                Units       = cf.Units('m'),
-    ...                                part        = [],
-    ...                                data        = numpy.array([[5, 6], [7, 8]])),
-    ...                      dtype=object),
-    ...          axes=[])
+    ...          numpy.array(Partition(
+    ...              location = [(0, 1), (2, 4)],
+    ...              shape = [1, 2],
+    ...              _dimensions = ['dim2', 'dim0'],
+    ...              Units = cf.Units('m'),
+    ...              part = [],
+    ...              data = numpy.array([[5, 6], [7, 8]])
+    ...          ), dtype=object),
+    ...          axes=[]
+    ...      )
 
         '''
         self.matrix = matrix
-        self.axes   = axes
-
+        self.axes = axes
 
     def __deepcopy__(self, memo):
         '''Used if copy.deepcopy is called on the variable.
 
-        ''' 
+        '''
         return self.copy()
 
-
     def __getitem__(self, indices):
         '''x.__getitem__(indices) <==> x[indices]
 
@@ -94,11 +95,11 @@ Attribute   Description
     dimensions are always removed from the output array, i.e. a
     partition rather than a partition matrix is returned if the output
     array has size 1.
-    
+
     Returns either a partition or a partition matrix.
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (5, 3)
     >>> pm[0, 1]
@@ -109,7 +110,7 @@ Attribute   Description
     (5,)
     >>> pm[1:4, slice(2, 0, -1)].shape
     (3, 2)
-    
+
     >>> pm.shape
     ()
     >>> pm[()]
@@ -125,11 +126,10 @@ Attribute   Description
 
         if out.size == 1:
             return self.matrix.item()
-        
-        axes = [axis for axis, n in zip(self.axes, out.shape) if n != 1]
-        
-        return type(self)(numpy_squeeze(out), axes)
 
+        axes = [axis for axis, n in zip(self.axes, out.shape) if n != 1]
+
+        return type(self)(numpy_squeeze(out), axes)
 
     def __repr__(self):
         '''x.__repr__() <==> repr(x)
@@ -137,7 +137,6 @@ Attribute   Description
         '''
         return '<CF %s: %s>' % (self.__class__.__name__, self.shape)
 
-
     def __setitem__(self, indices, value):
         '''x.__setitem__(indices, y) <==> x[indices]=y
 
@@ -146,19 +145,19 @@ Attribute   Description
     iterable. If a tuple of integers (or slices equivalent to an
     integer) is given then there must be one index per partition
     matrix dimension.
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (3,)
     >>> pm[2] = p1
     >>> pm[:] = [p1, p2, p3]
-    
+
     >>> pm.shape
     (2, 3)
     >>> pm[0, 2] = p1
-    
-    
+
+
     >>> pm.shape
     ()
     >>> pm[()] = p1
@@ -167,7 +166,6 @@ Attribute   Description
         '''
         self.matrix[indices] = value
 
-
     def __str__(self):
         '''x.__str__() <==> str(x)
 
@@ -179,22 +177,21 @@ Attribute   Description
 
         return '\n'.join(out)
 
-
     def change_axis_names(self, axis_map):
         '''Change the axis names.
 
     The axis names are arbitrary, so mapping them to another arbitrary
     collection does not change the data array values, units, nor axis
     order.
-    
+
     :Parameters:
-    
+
         axis_map: `dict`
-    
+
     :Returns:
-    
+
         `None`
-    
+
         '''
         # Partition dimensions
         axes = self.axes
@@ -206,7 +203,6 @@ Attribute   Description
         for partition in self.matrix.flat:
             partition.change_axis_names(axis_map)
 
-
     # ----------------------------------------------------------------
     # Attributes
     # ----------------------------------------------------------------
@@ -215,7 +211,7 @@ Attribute   Description
         '''A flat iterator over the partitions in the partition matrix.
 
     **Examples:**
-    
+
     >>> pm.shape
     [2, 2]
     >>> for partition in pm.flat:
@@ -225,10 +221,10 @@ Attribute   Description
     <CF Units: km hr-1>
     <CF Units: miles day-1>
     <CF Units: mm minute-1>
-    
+
     >>> pm.flat
     <numpy.flatiter at 0x1e14840>
-    
+
     >>> flat = pm.flat
     >>> next(flat)
     <Partition at 0x1934c80>
@@ -238,46 +234,44 @@ Attribute   Description
         '''
         return self.matrix.flat
 
-
     @property
     def ndim(self):
         '''The number of partition dimensions in the partition matrix.
 
     Not to be confused with the number of dimensions of the master
     data array.
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (8, 4)
     >>> pm.ndim
     2
-    
+
     >>> pm.shape
     ()
     >>> pm.ndim
     0
 
-        '''       
+        '''
         return self.matrix.ndim
 
-
     @property
     def shape(self):
         '''List of the partition matrix's dimension sizes.
-    
+
     Not to be confused with the sizes of the master data array's
     dimensions.
-    
+
     **Examples:**
-    
+
     >>> pm.ndim
     2
     >>> pm.size
     32
     >>> pm.shape
     (8, 4)
-    
+
     >>> pm.ndim
     0
     >>> pm.shape
@@ -286,21 +280,20 @@ Attribute   Description
         '''
         return self.matrix.shape
 
-
     @property
     def size(self):
         '''The number of partitions in the partition matrix.
 
     Not to be confused with the number of elements in the master data
     array.
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (8, 4)
     >>> pm.size
     32
-    
+
     >>> pm.shape
     ()
     >>> pm.size
@@ -309,69 +302,67 @@ Attribute   Description
         '''
         return self.matrix.size
 
-
     def add_partitions(self, adimensions, master_flip, extra_boundaries, axis):
         '''Add partition boundaries.
 
     :Parameters:
-    
+
         adimensions: `list`
             The ordered axis names of the master array.
-    
+
         master_flip: `list`
-    
+
         extra_boundaries: `list` of `int`
             The boundaries of the new partitions.
-    
+
         axis: `str`
             The name of the axis to have the new partitions.
-    
-        '''     
-        def _update_p(matrix, location, master_index, 
+
+        '''
+        def _update_p(matrix, location, master_index,
                       part, master_axis_to_position, master_flip):
             '''TODO
 
         :Parameters:
-    
+
             matrix: numpy array of `Partition` objects
-    
+
             location: `list`
-    
+
             master_index: `int`
-    
+
             part: `list`
-    
+
             master_axis_to_position: `dict`
-    
+
             master_flip: `list`
-    
+
         :Returns:
-    
+
             numpy array of `Partition` objects
-    
-            '''      
+
+            '''
             for partition in matrix.flat:
                 partition.location = partition.location[:]
-                partition.shape    = partition.shape[:]
+                partition.shape = partition.shape[:]
 
                 partition.location[master_index] = location
-                partition.shape[master_index]    = shape                       
+                partition.shape[master_index] = shape
 
                 partition.new_part(part,
                                    master_axis_to_position,
                                    master_flip)
-            #--- End: for
+            # --- End: for
 
             return matrix
 
-              
         # If no extra boundaries have been provided, just return
         # without doing anything
         if not extra_boundaries:
             return
 
         master_index = adimensions.index(axis)
-        index        = self.axes.index(axis)
+        index = self.axes.index(axis)
 
         # Find the position of the extra-boundaries dimension in the
         # list of master array dimensions
@@ -390,11 +381,11 @@ Attribute   Description
         new_shape = list(shape)
         new_shape[index] += len(extra_boundaries)
         new_matrix = numpy_empty(new_shape, dtype=object)
-        
-        part        = [slice(None)] * len(adimensions)
-        indices     = [slice(None)] * matrix.ndim
+
+        part = [slice(None)] * len(adimensions)
+        indices = [slice(None)] * matrix.ndim
         new_indices = indices[:]
-        new_indices[index] = slice(0, 1) #0
+        new_indices[index] = slice(0, 1)  # 0
 
         # Find the first extra boundary
         x = extra_boundaries.pop(0)
@@ -412,28 +403,28 @@ Attribute   Description
                 # straight into the new matrix
                 new_matrix[tuple(new_indices)] = sub_matrix
 #                new_indices[index] += 1
-                new_indices[index] = slice(new_indices[index].start+1, new_indices[index].stop+1)
+                new_indices[index] = slice(
+                    new_indices[index].start+1, new_indices[index].stop+1)
                 continue
 
             # --------------------------------------------------------
             # Still here? Then this new boundary *is* within the span
             # of this sub-matrix.
             # --------------------------------------------------------
-            
+
             # Find the new extent of the original partition(s)
             location = (r0, x)
-            shape    = x - r0
-            part[master_index]  = slice(0, shape)
-            
+            shape = x - r0
+            part[master_index] = slice(0, shape)
+
             # Create new partition(s) in place of the original ones(s)
             # and set the location, shape and part attributes
-            new_matrix[tuple(new_indices)] = _update_p(deepcopy(sub_matrix),
-                                                       location, master_index,
-                                                       part,
-                                                       master_axis_to_position,
-                                                       master_flip)
+            new_matrix[tuple(new_indices)] = _update_p(
+                deepcopy(sub_matrix), location, master_index, part,
+                master_axis_to_position, master_flip)
 #            new_indices[index] += 1
-            new_indices[index] = slice(new_indices[index].start+1, new_indices[index].stop+1)
+            new_indices[index] = slice(
+                new_indices[index].start+1, new_indices[index].stop+1)
 
             while x < r1:
                 # Find the extent of the new partition(s)
@@ -450,57 +441,56 @@ Attribute   Description
                     location1 = min(extra_boundaries[0], r1)
 
                 location = (x, location1)
-                shape    = location1 - x
-                offset   = x - r0
+                shape = location1 - x
+                offset = x - r0
                 part[master_index] = slice(offset, offset + shape)
 
                 # Create the new partition(s) and set the
-                # location, shape and part attributes 
-                new_matrix[tuple(new_indices)] = _update_p(deepcopy(sub_matrix),
-                                                           location, master_index,
-                                                           part,
-                                                           master_axis_to_position,
-                                                           master_flip)
+                # location, shape and part attributes
+                new_matrix[tuple(new_indices)] = _update_p(
+                    deepcopy(sub_matrix), location, master_index, part,
+                    master_axis_to_position, master_flip
+                )
 
-                new_indices[index] = slice(new_indices[index].start+1, new_indices[index].stop+1)
+                new_indices[index] = slice(
+                    new_indices[index].start+1, new_indices[index].stop+1)
 #                new_indices[index] += 1
 
                 if not extra_boundaries:
                     # There are no more extra boundaries, so we can
-                    # return now
-#                    new_indices[index] = slice(new_indices[index],
-#                                               None)
+                    # return now.
+                    #  new_indices[index] = slice(new_indices[index],
+                    #  None)
                     new_indices[index] = slice(new_indices[index].start,
                                                None)
-                    indices[index]     = slice(i+1, None)  
+                    indices[index] = slice(i+1, None)
 
                     new_matrix[tuple(new_indices)] = matrix[tuple(indices)]
                     self.matrix = new_matrix
 
                     return
-                                
+
                 # Still here? Then move on to the next new boundary
                 x = extra_boundaries.pop(0)
-            #--- End: while                                   
-        #--- End: for
+            # --- End: while
+        # --- End: for
 
         self.matrix = new_matrix
 
-
     def copy(self):
         '''Return a deep copy.
 
     ``pm.copy()`` is equivalent to ``copy.deepcopy(pm)``.
-    
+
     :Returns:
-    
+
             The deep copy.
-    
+
     **Examples:**
-    
+
     >>> pm.copy()
 
-        ''' 
+        '''
         # ------------------------------------------------------------
         # NOTE: 15 May 2013. It is necesary to treat
         #       self.matrix.ndim==0 as a special case since there is a
@@ -510,16 +500,15 @@ Attribute   Description
         matrix = self.matrix
 
         if not matrix.ndim:
-            new_matrix = _empty_matrix.copy() #numpy_empty((), dtype=object)
+            new_matrix = _empty_matrix.copy()  # numpy_empty((), dtype=object)
             new_matrix[()] = matrix.item().copy()
-            return type(self)(new_matrix , [])
+            return type(self)(new_matrix, [])
         else:
             new_matrix = numpy.empty(matrix.size, dtype=object)
             new_matrix[...] = [partition.copy() for partition in matrix.flat]
-            new_matrix.resize(matrix.shape)        
+            new_matrix.resize(matrix.shape)
             return type(self)(new_matrix, self.axes)
 
-
     # 0
     @_inplace_enabled
     def insert_dimension(self, axis, inplace=False):
@@ -527,20 +516,20 @@ Attribute   Description
 
     The new axis is always inserted at position 0, i.e. it becomes the
     new slowest varying axis.
-    
+
     .. seealso:: `flip`, `squeezes`, `swapaxes`, `transpose`
-    
+
     :Parameters:
-    
+
         axis: `str`
             The internal identity of the new axis.
-    
+
     :Returns:
-    
+
         `PartitionMatrix`
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (2, 3)
     >>> pm.insert_dimension('dim2')
@@ -551,21 +540,20 @@ Attribute   Description
         p = _inplace_enabled_define_and_cleanup(self)
 
         p.matrix = numpy_expand_dims(p.matrix, 0)
-        p.axes  = [axis] + p.axes
+        p.axes = [axis] + p.axes
 
         return p
 
-
     def ndenumerate(self):
         '''Return an iterator yielding pairs of array indices and values.
 
     :Returns:
-    
+
         `numpy.ndenumerate`
             An iterator over the array coordinates and values.
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (2, 3)
     >>> for i, partition in pm.ndenumerate():
@@ -581,22 +569,21 @@ Attribute   Description
         '''
         return numpy_ndenumerate(self.matrix)
 
-
     # 0
     def partition_boundaries(self, data_axes):
         '''Return the partition boundaries for each dimension.
 
     :Parameters:
-    
+
         data_axes: sequence
-    
+
     :Returns:
-    
+
         `dict`
-    
-        '''            
+
+        '''
         boundaries = {}
-        
+
         matrix = self.matrix
         indices = [0] * self.ndim
 
@@ -605,43 +592,42 @@ Attribute   Description
             j = data_axes.index(axis)
 
             sub_matrix = matrix[tuple(indices)]
-            
+
             b = [partition.location[j][0] for partition in sub_matrix.flat]
 
             # Python3: can't access variables from within previous
             # list comprehension
             last_partition = sub_matrix.flat[-1]
-            
+
             b.append(last_partition.location[j][1])
             boundaries[axis] = b
 
             indices[i] = 0
-        #--- End: for
+        # --- End: for
 
         return boundaries
 
-
     # 0
     @_inplace_enabled
     def swapaxes(self, axis0, axis1, inplace=False):
         '''Swap the positions of two axes.
 
     Note that this does not change the master data array.
-    
+
     .. seealso:: `insert_dimension`, `flip`, `squeeze`, `transpose`
-    
+
     :Parameters:
-    
+
         axis0, axis1: `int`, `int`
             Select the axes to swap. Each axis is identified by its
             original integer position.
-    
+
     :Returns:
-    
+
         `PartitionMatrix`
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (2, 3, 4, 5)
     >>> pm.swapaxes(1, 2)
@@ -661,45 +647,44 @@ Attribute   Description
 
         return p
 
-
     # 0
     def set_location_map(self, data_axes, ns=None):
         '''Set the `!location` attribute of each partition of the partition
     matrix in place.
 
     :Parameters:
-    
+
         data_axes: sequence
             The axes of the master data array.
-    
+
         ns: sequence of `int`, optional
-    
+
     :Returns:
-    
+
         `None`
-            
+
     **Examples:**
-    
+
     >>> pm.set_location_map(['dim1', 'dim0'])
     >>> pm.set_location_map([])
 
         '''
         matrix = self.matrix
-        
+
         shape = matrix.shape
-        axes  = self.axes
+        axes = self.axes
 
         slice_None = slice(None)
-        
+
         indices = [slice_None] * matrix.ndim
-        
+
         # Never update location in-place
         for partition in matrix.flat:
             partition.location = partition.location[:]
- 
+
         if ns is None:
             ns = range(len(data_axes))
-       
+
         for axis, n in zip(data_axes, ns):
 
             if axis in axes:
@@ -709,7 +694,7 @@ Attribute   Description
                 m = axes.index(axis)
                 start = 0
                 for i in range(shape[m]):
-                    indices[m] = slice(i, i+1) # i
+                    indices[m] = slice(i, i+1)  # i
                     flat = matrix[tuple(indices)].flat
 
                     partition = next(flat)
@@ -720,10 +705,10 @@ Attribute   Description
 
                     for partition in flat:
                         partition.location[n] = location
-                    
+
                     start = stop
-                    
-                #--- End: for
+
+                # --- End: for
                 indices[m] = slice_None
             else:
                 # ----------------------------------------------------
@@ -736,9 +721,8 @@ Attribute   Description
                 partition.location[n] = location
 
                 for partition in flat:
-                    partition.location[n] = location 
-        #--- End: for
-
+                    partition.location[n] = location
+        # --- End: for
 
     # 0
     @_inplace_enabled
@@ -746,21 +730,21 @@ Attribute   Description
         '''Remove all size 1 axes.
 
     Note that this does not change the master data array.
-    
+
     .. seealso:: `insert_dimension`, `flip`, `swapaxes`, `transpose`
-    
+
     :Returns:
-    
+
         `PartitionMatrix`
-    
+
     **Examples:**
-    
+
     >>> pm.shape
     (1, 2, 1, 2)
     >>> pm.squeeze()
     >>> pm.shape
     (2, 2)
-    
+
     >>> pm.shape
     (1,)
     >>> pm.squeeze()
@@ -774,36 +758,35 @@ Attribute   Description
         p = _inplace_enabled_define_and_cleanup(self)
 
         matrix = p.matrix
-        shape  = matrix.shape
+        shape = matrix.shape
 
         if 1 in shape:
-            p.matrix = matrix.squeeze()        
+            p.matrix = matrix.squeeze()
 
             axes = p.axes
             p.axes = [axis for axis, size in zip(axes, shape) if size > 1]
 
         return p
 
-
     @_inplace_enabled
     def transpose(self, axes, inplace=False):
         '''Permute the partition dimensions of the partition matrix in place.
 
     Note that this does not change the master data array.
-    
+
     .. seealso:: `insert_dimension`, `flip`, `squeeze`, `swapaxes`
-    
+
     :Parameters:
-    
-        axes: sequence of `int` 
+
+        axes: sequence of `int`
             Permute the axes according to the values given.
-    
+
     :Returns:
-    
+
         `PartitionMatrix`
-    
+
     **Examples:**
-    
+
     >>> pm.ndim
     3
     >>> pm.transpose((2, 0, 1))
@@ -819,7 +802,6 @@ Attribute   Description
 
         return p
 
-
     # ----------------------------------------------------------------
     # Deprecated attributes and methods
     # ----------------------------------------------------------------
@@ -830,8 +812,9 @@ Attribute   Description
     instead.
 
         '''
-        _DEPRECATION_ERROR_METHOD(self, 'expand_dims',
-                                  "Use method 'insert_dimension' instead.") # pragma: no cover
+        _DEPRECATION_ERROR_METHOD(
+            self, 'expand_dims', "Use method 'insert_dimension' instead."
+        )  # pragma: no cover
 
-             
-#--- End: class
+
+# --- End: class
