@@ -11,17 +11,18 @@ from .units        import Units
 from .data.data import Data
 
 from . import mixin
-from . import abstract
+#from . import abstract
 
 from .functions import (_DEPRECATION_ERROR_KWARGS,
-                        _DEPRECATION_ERROR_ATTRIBUTE,
-                        )
+                        _DEPRECATION_ERROR_ATTRIBUTE)
 
-from .decorators import _deprecation_error_i_kwarg
+from .decorators import (_inplace_enabled,
+                         _inplace_enabled_define_and_cleanup,
+                         _deprecated_kwarg_check)
 
 
-class DimensionCoordinate(abstract.Coordinate,
-                          mixin.PropertiesDataBounds,                          
+class DimensionCoordinate(mixin.Coordinate,
+                          mixin.PropertiesDataBounds,
                           cfdm.DimensionCoordinate):
     '''A dimension coordinate construct of the CF data model.
 
@@ -36,7 +37,7 @@ class DimensionCoordinate(abstract.Coordinate,
     cell must have exactly two vertices. CF-netCDF coordinate
     variables and numeric scalar coordinate variables correspond to
     dimension coordinate constructs.
-    
+
     The dimension coordinate construct consists of a data array of the
     coordinate values which spans a subset of the domain axis
     constructs, an optional array of cell bounds recording the extents
@@ -49,9 +50,9 @@ class DimensionCoordinate(abstract.Coordinate,
     domain. Note that, for climatological time axes, the bounds are
     interpreted in a special way indicated by the cell method
     constructs.
-    
+
     **NetCDF interface**
-    
+
     The netCDF variable name of the construct may be accessed with the
     `nc_set_variable`, `nc_get_variable`, `nc_del_variable` and
     `nc_has_variable` methods.
@@ -65,7 +66,6 @@ class DimensionCoordinate(abstract.Coordinate,
         '''
         return super().__repr__().replace('<', '<CF ', 1)
 
-
     def _centre(self, period):
         '''It assumed, but not checked, that the period has been set.
 
@@ -76,9 +76,8 @@ class DimensionCoordinate(abstract.Coordinate,
             mx = self.data[-1]
         else:
             mx = self.data[0]
-            
-        return ((mx // period) * period).squeeze()
 
+        return ((mx // period) * period).squeeze()
 
     def _infer_direction(self):
         '''Return True if a coordinate is increasing, otherwise return False.
@@ -86,21 +85,21 @@ class DimensionCoordinate(abstract.Coordinate,
     A coordinate is considered to be increasing if its *raw* data
     array values are increasing in index space or if it has no data
     not bounds data.
-    
+
     If the direction can not be inferred from the coordinate's data
     then the coordinate's units are used.
-    
+
     The direction is inferred from the coordinate's data array values
     or its from coordinates. It is not taken directly from its
     `cf.Data` object.
-    
+
     :Returns:
-    
+
         `bool`
             Whether or not the coordinate is increasing.
-            
+
     **Examples:**
-    
+
     >>> c.array
     array([  0  30  60])
     >>> c._get_direction()
@@ -120,20 +119,19 @@ class DimensionCoordinate(abstract.Coordinate,
             if data._size > 1:
                 data = data[0:2].array
                 return bool(data.item(0,) < data.item(1,))
-        #--- End: if
+        # --- End: if
 
         # Still here?
         data = self.get_bounds_data(None)
         if data is not None:
             # Infer the direction from the dimension coordinate's
             # bounds
-            b  = data[(0,)*(data.ndim-1)].array
+            b = data[(0,)*(data.ndim-1)].array
             return bool(b.item(0,) < b.item(1,))
 
         # Still here? Then infer the direction from the units.
         return not self.Units.ispressure
 
-
     # ----------------------------------------------------------------
     # Attributes
     # ----------------------------------------------------------------
@@ -141,29 +139,29 @@ class DimensionCoordinate(abstract.Coordinate,
 #    def cellsize(self):
 #        '''The cell sizes.
 #
-#:Returns:
-#        
-#    `Data`
-#        The size for each cell.
+#    :Returns:
 #
-#**Examples:**
+#        `Data`
+#            The size for each cell.
 #
-#>>> print(c.bounds)
-#<CF Bounds: latitude(3, 2) degrees_north>
-#>>> print(c.bounds.array)
-#[[-90. -87.]
-# [-87. -80.]
-# [-80. -67.]]
-#>>> print(d.cellsize)
-#<CF Data(3): [3.0, 7.0, 13.0] degrees_north>
-#>>> print(d.cellsize.array)
-#[  3.   7.  13.]
-#>>> print(c.sin().cellsize.array)
-#[ 0.00137047  0.01382178  0.0643029 ]
+#    **Examples:**
 #
-#>>> del(c.bounds)
-#>>> c.cellsize
-#AttributeError: Can't get cell sizes when coordinates have no bounds
+#    >>> print(c.bounds)
+#    <CF Bounds: latitude(3, 2) degrees_north>
+#    >>> print(c.bounds.array)
+#    [[-90. -87.]
+#     [-87. -80.]
+#     [-80. -67.]]
+#    >>> print(d.cellsize)
+#    <CF Data(3): [3.0, 7.0, 13.0] degrees_north>
+#    >>> print(d.cellsize.array)
+#    [  3.   7.  13.]
+#    >>> print(c.sin().cellsize.array)
+#    [ 0.00137047  0.01382178  0.0643029 ]
+#
+#    >>> del(c.bounds)
+#    >>> c.cellsize
+#    AttributeError: Can't get cell sizes when coordinates have no bounds
 #
 #        '''
 #        cells = self.get_bounds_data(None)
@@ -171,36 +169,35 @@ class DimensionCoordinate(abstract.Coordinate,
 #            raise AttributeError(
 #                "Can't get cell sizes when coordinates have no bounds")
 #
-#        if self.direction():            
+#        if self.direction():
 #            cells = cells[:, 1] - cells[:, 0]
 #        else:
 #            cells = cells[:, 0] - cells[:, 1]
 #
 #        cells.squeeze(1, inplace=True)
-#        
+#
 #        return cells
 
-           
     @property
-    def decreasing(self): 
+    def decreasing(self):
         '''True if the dimension coordinate is decreasing, otherwise False.
 
     A dimension coordinate is increasing if its coordinate values are
     increasing in index space.
-    
+
     The direction is inferred from one of, in order of precedence:
-    
+
     * The data array
     * The bounds data array
     * The `units` CF property
-    
+
     :Returns:
-    
+
         `bool`
             Whether or not the coordinate is decreasing.
 
     **Examples:**
-    
+
     >>> c.decreasing
     False
     >>> c.flip().increasing
@@ -209,27 +206,26 @@ class DimensionCoordinate(abstract.Coordinate,
         '''
         return not self.direction()
 
-
     @property
-    def increasing(self): 
+    def increasing(self):
         '''`True` for dimension coordinate constructs, `False` otherwise.
 
     A dimension coordinate is increasing if its coordinate values are
     increasing in index space.
-    
+
     The direction is inferred from one of, in order of precedence:
-    
+
     * The data array
     * The bounds data array
     * The `units` CF property
-    
+
     :Returns:
-    
+
         `bool`
             Whether or not the coordinate is increasing.
 
     **Examples:**
-    
+
     >>> c.decreasing
     False
     >>> c.flip().increasing
@@ -238,44 +234,43 @@ class DimensionCoordinate(abstract.Coordinate,
         '''
         return self.direction()
 
-
     @property
-    def isdimension(self): 
+    def isdimension(self):
         '''True, denoting that the variable is a dimension coordinate object.
 
     .. seealso::`isauxiliary`, `isdomainancillary`, `isfieldancillary`,
                 `ismeasure`
-    
+
     **Examples:**
-    
+
     >>> c.isdimension
     True
 
         '''
         return True
 
-
 #    @property
 #    def lower_bounds(self):
 #        '''The lower dimension coordinate bounds in a `cf.Data` object.
 #
-#.. seealso:: `bounds`, `upper_bounds`
+#    .. seealso:: `bounds`, `upper_bounds`
 #
-#**Examples:**
+#    **Examples:**
 #
-#>>> print(c.bounds.array)
-#[[ 5  3]
-# [ 3  1]
-# [ 1 -1]]
-#>>> c.lower_bounds
-#<CF Data(3): [3, 1, -1]>
-#>>> print(c.lower_bounds.array)
-#[ 3  1 -1]
+#    >>> print(c.bounds.array)
+#    [[ 5  3]
+#     [ 3  1]
+#     [ 1 -1]]
+#    >>> c.lower_bounds
+#    <CF Data(3): [3, 1, -1]>
+#    >>> print(c.lower_bounds.array)
+#    [ 3  1 -1]
 #
 #        '''
 #        data = self.get_bounds_data(None)
 #        if data is None:
-#            raise ValueError("Can't get lower bounds when there are no bounds")
+#            raise ValueError(
+#                "Can't get lower bounds when there are no bounds")
 #
 #        if self.direction():
 #            i = 0
@@ -291,23 +286,24 @@ class DimensionCoordinate(abstract.Coordinate,
 #    def upper_bounds(self):
 #        '''The upper dimension coordinate bounds in a `cf.Data` object.
 #
-#.. seealso:: `bounds`, `lower_bounds`
+#    .. seealso:: `bounds`, `lower_bounds`
 #
-#**Examples:**
+#    **Examples:**
 #
-#>>> print(c.bounds.array)
-#[[ 5  3]
-# [ 3  1]
-# [ 1 -1]]
-#>>> c.upper_bounds      
-#<CF Data(3): [5, 3, 1]>
-#>>> print(c.upper_bounds.array)
-#[5  3  1]
+#    >>> print(c.bounds.array)
+#    [[ 5  3]
+#     [ 3  1]
+#     [ 1 -1]]
+#    >>> c.upper_bounds
+#    <CF Data(3): [5, 3, 1]>
+#    >>> print(c.upper_bounds.array)
+#    [5  3  1]
 #
 #        '''
 #        data = self.get_bounds_data(None)
 #        if data is None:
-#            raise ValueError("Can't get upper bounds when there are no bounds")
+#            raise ValueError(
+#                "Can't get upper bounds when there are no bounds")
 #
 #        if self.direction():
 #            i = 1
@@ -316,32 +312,31 @@ class DimensionCoordinate(abstract.Coordinate,
 #
 #        return data[..., i].squeeze(1)
 
-
     def direction(self):
         '''Return True if the dimension coordinate values are increasing,
     otherwise return False.
-    
+
     Dimension coordinates values are increasing if its coordinate
     values are increasing in index space.
-    
+
     The direction is inferred from one of, in order of precedence:
-    
+
     * The data array
     * The bounds data array
     * The `units` CF property
-    
+
     :Returns:
-    
+
         `bool`
             Whether or not the coordinate is increasing.
-            
+
     **Examples:**
-    
+
     >>> c.array
     array([  0  30  60])
     >>> c.direction()
     True
-    
+
     >>> c.bounds.array
     array([  30  0])
     >>> c.direction()
@@ -357,57 +352,56 @@ class DimensionCoordinate(abstract.Coordinate,
 
         return _direction
 
-
     def create_bounds(self, bound=None, cellsize=None, flt=0.5,
                       max=None, min=None):
         '''Create cell bounds.
-    
+
     :Parameters:
-    
+
         bound: optional
             If set to a value larger (smaller) than the largest
             (smallest) coordinate value then bounds are created which
             include this value and for which each coordinate is in the
             centre of its bounds. Ignored if *create* is False.
-    
+
         cellsize: optional
             Define the exact size of each cell that is
             created. Created cells are allowed to overlap do not have
             to be contigious.  Ignored if *create* is False. The
             *cellsize* parameter may be one of:
-    
+
               * A data-like scalar (see below) that defines the cell size,
                 either in the same units as the coordinates or in the
                 units provided. Note that in this case, the position of
                 each coordinate within the its cell is controlled by the
                 *flt* parameter.
-    
-                *Parameter example:*     
+
+                *Parameter example:*
                     To specify cellsizes of 10, in the same units as the
                     coordinates: ``cellsize=10``.
-        
+
                 *Parameter example:*
                     To specify cellsizes of 1 day: ``cellsize=cf.Data(1,
                     'day')`` (see `cf.Data` for details).
-        
+
                 *Parameter example:*
                      For coordinates ``1, 2, 10``, setting ``cellsize=1``
                      will result in bounds of ``(0.5, 1.5), (1.5, 2.5),
                      (9.5, 10.5)``.
-          
+
                 *Parameter example:*
                      For coordinates ``1, 2, 10`` kilometres, setting
                      ``cellsize=cf.Data(5000, 'm')`` will result in bounds
                      of ``(-1.5, 3.5), (-0.5, 4.5), (7.5, 12.5)`` (see
                      `cf.Data` for details).
-          
+
                 *Parameter example:*
                   For decreasing coordinates ``2, 0, -12`` setting,
                   ``cellsize=2`` will result in bounds of ``(3, 1),
                   (1, -1), (-11, -13)``.
-    
+
             ..
-    
+
               * A `cf.TimeDuration` defining the cell size. Only
                 applicable to reference time coordinates. It is possible
                 to "anchor" the cell bounds via the `cf.TimeDuration`
@@ -415,7 +409,7 @@ class DimensionCoordinate(abstract.Coordinate,
                 calendar month, starting and ending on the 15th day:
                 ``cellsize=cf.M(day=15)`` (see `cf.M` for details). Note
                 that the *flt* parameter is ignored in this case.
-          
+
                 *Parameter example:*
                   For coordinates ``1984-12-01 12:00, 1984-12-02
                   12:00, 2000-04-15 12:00`` setting,
@@ -423,14 +417,14 @@ class DimensionCoordinate(abstract.Coordinate,
                   ``(1984-12-01, 1984-12-02), (1984-12-02,
                   1984-12-03), (2000-05-15, 2000-04-16)`` (see `cf.D`
                   for details).
-    
+
                 *Parameter example:*
                   For coordinates ``1984-12-01, 1984-12-02,
                   2000-04-15`` setting, ``cellsize=cf.D()`` will
                   result in bounds of ``(1984-12-01, 1984-12-02),
                   (1984-12-02, 1984-12-03), (2000-05-15, 2000-04-16)``
                   (see `cf.D` for details).
-    
+
                 *Parameter example:*
                   For coordinates ``1984-12-01, 1984-12-02,
                   2000-04-15`` setting, ``cellsize=cf.D(hour=12)``
@@ -438,25 +432,25 @@ class DimensionCoordinate(abstract.Coordinate,
                   1984-12-01 12:00), (1984-12-01 12:00, 1984-12-02
                   12:00), (2000-05-14 12:00, 2000-04-15 12:00)`` (see
                   `cf.D` for details).
-    
+
                 *Parameter example:*
                   For coordinates ``1984-12-16 12:00, 1985-01-16
                   12:00`` setting, ``cellsize=cf.M()`` will result in
                   bounds of ``(1984-12-01, 1985-01-01), (1985-01-01,
                   1985-02-01)`` (see `cf.M` for details).
-    
+
                 *Parameter example:*
                   For coordinates ``1984-12-01 12:00, 1985-01-01
                   12:00`` setting, ``cellsize=cf.M()`` will result in
                   bounds of ``(1984-12-01, 1985-01-01), (1985-01-01,
                   1985-02-01)`` (see `cf.M` for details).
-    
+
                 *Parameter example:*
                   For coordinates ``1984-12-01 12:00, 1985-01-01
                   12:00`` setting, ``cellsize=cf.M(day=20)`` will
                   result in bounds of ``(1984-11-20, 1984-12-20),
                   (1984-12-20, 1985-01-20)`` (see `cf.M` for details).
-    
+
                 *Parameter example:*
                   For coordinates ``1984-03-01, 1984-06-01`` setting,
                   ``cellsize=cf.Y()`` will result in bounds of
@@ -466,75 +460,77 @@ class DimensionCoordinate(abstract.Coordinate,
                   because ``cf.Y()`` is equivalent to ``cf.Y(month=1,
                   day=1)`` and the closest 1st January to both
                   coordinates is 1st January 1984.
-    
+
             {+data-like-scalar} TODO
-    
+
         flt: `float`, optional
             When creating cells with sizes specified by the *cellsize*
             parameter, define the fraction of the each cell which is
             less its coordinate value. By default *flt* is 0.5, so that
             each cell has its coordinate at it's centre. Ignored if
             *cellsize* is not set.
-    
+
             *Parameter example:*
                For coordinates ``1, 2, 10``, setting ``cellsize=1,
                flt=0.5`` will result in bounds of ``(0.5, 1.5), (1.5,
                2.5), (9.5, 10.5)``.
-      
+
             *Parameter example:*
                For coordinates ``1, 2, 10``, setting ``cellsize=1,
                flt=0.25`` will result in bounds of ``(0.75, 1.75),
                (1.75, 2.75), (9.75, 10.75)``.
-      
-            *Parameter example:* 
+
+            *Parameter example:*
                For decreasing coordinates ``2, 0, -12``, setting
                ``cellsize=6, flt=0.9`` will result in bounds of
                ``(2.6, -3.4), (0.6, -5.4), (-11.4, -17.4)``.
-    
+
         min: optional
             Limit the created bounds to be no less than this number.
-    
-            *Parameter example:* 
+
+            *Parameter example:*
                To ensure that all latitude bounds are at least -90:
                ``min=-90``.
-    
+
         max: optional
             Limit the created bounds to be no more than this number.
-    
-            *Parameter example:* 
+
+            *Parameter example:*
                To ensure that all latitude bounds are at most 90:
                ``max=90``.
-    
+
         copy: `bool`, optional
             If `False` then the returned bounds are not independent of
             the existing bounds, if any, or those inserted, if
             *create* and *insert* are both True. By default the
             returned bounds are independent.
-    
+
     :Returns:
-    
+
         `Bounds` or `None`
             TODO
-    
+
     **Examples:**
-    
+
     >>> c.create_bounds()
     >>> c.create_bounds(bound=-9000.0)
 
         '''
         array = self.array
-        size = array.size    
+        size = array.size
 
         if cellsize is not None:
             if bound:
                 raise ValueError(
-                    "bound parameter can't be True when setting the cellsize parameter")
+                    "bound parameter can't be True when setting the "
+                    "cellsize parameter"
+                )
 
             if not isinstance(cellsize, TimeDuration):
                 # ----------------------------------------------------
                 # Create bounds based on cell sizes defined by a
                 # data-like object
-                # 
+                #
                 # E.g. cellsize=10
                 #      cellsize=cf.Data(1, 'day')
                 # ----------------------------------------------------
@@ -549,12 +545,12 @@ class DimensionCoordinate(abstract.Coordinate,
                             raise ValueError("jhgsjhbd jh ")
                         cellsize.Units = self.Units
                 cellsize = cellsize.datum()
-                
+
                 cellsize0 = cellsize * flt
                 cellsize1 = cellsize * (1 - flt)
                 if not self.direction():
                     cellsize0, cellsize1 = -cellsize1, -cellsize0
-                
+
                 bounds = numpy_empty((size, 2), dtype=array.dtype)
                 bounds[:, 0] = array - cellsize0
                 bounds[:, 1] = array + cellsize1
@@ -562,7 +558,7 @@ class DimensionCoordinate(abstract.Coordinate,
                 # ----------------------------------------------------
                 # Create bounds based on cell sizes defined by a
                 # TimeDuration object
-                # 
+                #
                 # E.g. cellsize=cf.s()
                 #      cellsize=cf.m()
                 #      cellsize=cf.h()
@@ -576,8 +572,10 @@ class DimensionCoordinate(abstract.Coordinate,
                 # ----------------------------------------------------
                 if not self.Units.isreftime:
                     raise ValueError(
-                        "Can't create reference time bounds for non-reference time coordinates: {0!r}".format(
-                            self.Units))
+                        "Can't create reference time bounds for "
+                        "non-reference time coordinates: {0!r}".format(
+                            self.Units)
+                    )
 
                 bounds = numpy_empty((size, 2), dtype=object)
 
@@ -599,9 +597,9 @@ class DimensionCoordinate(abstract.Coordinate,
                 bounds_1d = [array.item(0,)*1.5 - array.item(1,)*0.5]
                 bounds_1d.extend((array[0:-1] + array[1:])*0.5)
                 bounds_1d.append(array.item(-1,)*1.5 - array.item(-2,)*0.5)
-    
+
                 dtype = type(bounds_1d[0])
-    
+
                 if max is not None:
                     if self.direction():
                         bounds_1d[-1] = max
@@ -612,7 +610,7 @@ class DimensionCoordinate(abstract.Coordinate,
                         bounds_1d[0] = min
                     else:
                         bounds_1d[-1] = min
-                        
+
             else:
                 # ----------------------------------------------------
                 # Create
@@ -620,7 +618,7 @@ class DimensionCoordinate(abstract.Coordinate,
                 direction = self.direction()
                 if not direction and size > 1:
                     array = array[::-1]
-    
+
                 bounds_1d = [bound]
                 if bound <= array.item(0,):
                     for i in range(size):
@@ -630,68 +628,64 @@ class DimensionCoordinate(abstract.Coordinate,
                     for i in range(size-1, -1, -1):
                         bound = 2.0*array.item(i,) - bound
                         bounds_1d.append(bound)
-    
+
                     bounds_1d = bounds_1d[::-1]
                 else:
                     raise ValueError("bad bound value")
-    
+
                 dtype = type(bounds_1d[-1])
-    
-                if not direction:               
+
+                if not direction:
                     bounds_1d = bounds_1d[::-1]
-            #--- End: if
+            # --- End: if
 
             bounds = numpy_empty((size, 2), dtype=dtype)
-            bounds[:,0] = bounds_1d[:-1]
-            bounds[:,1] = bounds_1d[1:]        
-        #--- End: if
+            bounds[:, 0] = bounds_1d[:-1]
+            bounds[:, 1] = bounds_1d[1:]
+        # --- End: if
 
         # Create coordinate bounds object
         bounds = Bounds(data=Data(bounds, units=self.Units),
                         copy=False)
 
-        return bounds            
+        return bounds
 
-
-    @_deprecation_error_i_kwarg
+    @_deprecated_kwarg_check('i')
+    @_inplace_enabled
     def flip(self, axes=None, inplace=False, i=False):
         '''TODO
         '''
-        d = super().flip(axes=axes, inplace=inplace)
-        if inplace:
-            d = self
+        d = _inplace_enabled_define_and_cleanup(self)
+        super(d.__class__, d).flip(axes=axes, inplace=True)
 
         direction = d._custom.get('direction')
         if direction is not None:
             d._custom['direction'] = not direction
 
-        if inplace:
-            d = None
         return d
 
-            
     def get_bounds(self, default=ValueError(), **kwargs):
         '''Return the bounds.
 
     .. versionadded:: 3.0.0
-    
+
     .. seealso:: `bounds`, `create_bounds', `get_data`, `del_bounds`,
                  `has_bounds`, `set_bounds`
-    
-    :Parameters:	
-    
+
+    :Parameters:
+
         default: optional
             Return the value of the default parameter if bounds have
             not been set. If set to an `Exception` instance then it
             will be raised instead.
-    
-    :Returns:	
-    
+
+    :Returns:
+
         `Bounds`
             The bounds.
 
     **Examples:**
-    
+
     >>> b = Bounds(data=cfdm.Data(range(10).reshape(5, 2)))
     >>> c.set_bounds(b)
     >>> c.has_bounds()
@@ -708,34 +702,35 @@ class DimensionCoordinate(abstract.Coordinate,
     >>> print(c.del_bounds(None))
     None
 
-        '''        
+        '''
         if kwargs:
             _DEPRECATION_ERROR_KWARGS(
                 self, 'get_bounds', kwargs,
-                "Bounds creation now uses the 'create_bounds' and 'set_bounds' methods.") # pragma: no cover
-            
-        return super().get_bounds(default=default)
+                "Bounds creation now uses the 'create_bounds' and "
+                "'set_bounds' methods."
+            )  # pragma: no cover
 
+        return super().get_bounds(default=default)
 
     def period(self, *value):
         '''Set the period for cyclic coordinates.
 
     :Parameters:
-    
+
         value: data-like or `None`, optional
             The period. The absolute value is used.
-    
+
             {+data-like-scalar} TODO
-    
+
     :Returns:
-    
+
         out: `cf.Data` or `None`
             The period prior to the change, or the current period if
             no *value* was specified. In either case, None is returned
             if the period had not been set previously.
-    
+
     **Examples:**
-    
+
     >>> print(c.period())
     None
     >>> c.Units
@@ -758,14 +753,14 @@ class DimensionCoordinate(abstract.Coordinate,
     >>> c.period()
     <CF Data(): 360.0 degrees_east>
 
-        '''     
+        '''
         old = self._custom.get('period')
         if old is not None:
             old = old.copy()
 
         if not value:
             return old
-  
+
         value = value[0]
 
         if value is not None:
@@ -778,21 +773,23 @@ class DimensionCoordinate(abstract.Coordinate,
                     value.Units = self.Units
                 else:
                     raise ValueError(
-                        "Period units {!r} are not equivalent to coordinate units {!r}".format(
-                            units, self.Units))
-            #--- End: if
+                        "Period units {!r} are not equivalent to coordinate "
+                        "units {!r}".format(units, self.Units)
+                    )
+            # --- End: if
 
             value = abs(value)
             value.dtype = float
-            
+
             array = self.array
-            r =  abs(array[-1] - array[0])
+            r = abs(array[-1] - array[0])
 
             if r >= value.datum(0):
                 raise ValueError(
-                    "The coordinate range of {!r} is not less than the period of {!r}".format(
-                        r, value))
-        #--- End: if
+                    "The coordinate range of {!r} is not less than the "
+                    "period of {!r}".format(r, value)
+                )
+        # --- End: if
 
         self._custom['period'] = value
 
@@ -805,58 +802,64 @@ class DimensionCoordinate(abstract.Coordinate,
     grid longitude) dimension coordinate construct with bounds and the
     first and last bounds values differ by 360 degrees (or an
     equivalent amount in other units).
-       
+
     .. versionadded:: 3.0.0
-    
+
     .. seealso:: `isperiodic`, `period`
-    
+
     :Parameters:
 
         verbose: `bool`, optional
             TODO
 
     :Returns:
-    
+
        `bool`
-    
+
     **Examples:**
-    
+
     >>> f.autocyclic()
 
         '''
         if not self.Units.islongitude:
-            if verbose: print(0)
-            if self.get_property('standard_name', None) not in ('longitude', 'grid_longitude'):
-                if verbose: print(1)
+            if verbose:
+                print(0)
+            if (self.get_property('standard_name', None) not in
+                    ('longitude', 'grid_longitude')):
+                if verbose:
+                    print(1)
                 return False
-        #--- End: if
-        
+        # --- End: if
+
         bounds = self.get_bounds(None)
         if bounds is None:
-            if verbose: print(2)
+            if verbose:
+                print(2)
             return False
 
         bounds_data = bounds.get_data(None)
         if bounds_data is None:
-            if verbose: print(3)
+            if verbose:
+                print(3)
             return False
 
         bounds = bounds_data.array
-        
+
         period = Data(360.0, units='degrees')
 
         period.Units = bounds_data.Units
 
         if abs(bounds[-1, -1] - bounds[0, 0]) != period.array:
-            if verbose: print(4)
+            if verbose:
+                print(4)
             return False
 
         self.period(period)
 
         return True
-    
 
-    @_deprecation_error_i_kwarg
+    @_deprecated_kwarg_check('i')
+    @_inplace_enabled
     def roll(self, axis, shift, inplace=False, i=False):
         '''TODO
         '''
@@ -865,7 +868,7 @@ class DimensionCoordinate(abstract.Coordinate,
                 return
             else:
                 return self.copy()
-        #--- End: if
+        # --- End: if
 
         shift %= self.size
 
@@ -881,26 +884,26 @@ class DimensionCoordinate(abstract.Coordinate,
             raise ValueError(
                 "Can't roll {} array when no period has been set".format(
                     self.__class__.__name__))
-        
+
         direction = self.direction()
 
         centre = self._centre(period)
 
         if axis not in [0, -1]:
-            raise ValueError("Can't roll axis {} when there is only one axis".format(axis))
-        
-        c = super().roll(axis, shift, inplace=inplace)
-        if inplace:
-            c = self
-    
-        c.dtype = numpy_result_type(c.dtype, period.dtype)        
+            raise ValueError(
+                "Can't roll axis {} when there is only one axis".format(axis))
+
+        c = _inplace_enabled_define_and_cleanup(self)
+        super(c.__class__, c).roll(axis, shift, inplace=True)
+
+        c.dtype = numpy_result_type(c.dtype, period.dtype)
 
         b = c.get_bounds(None)
         bounds_data = b.get_data(None)
         if bounds_data is not None:
             b.dtype = numpy_result_type(bounds_data.dtype, period.dtype)
             bounds_data = b.get_data(None)
-        
+
         if direction:
             # Increasing
             c[:shift] -= period
@@ -910,7 +913,7 @@ class DimensionCoordinate(abstract.Coordinate,
             if c.data[0] <= centre - period:
                 c += period
                 if bounds_data is not None:
-                    b += period 
+                    b += period
         else:
             # Decreasing
             c[:shift] += period
@@ -921,15 +924,12 @@ class DimensionCoordinate(abstract.Coordinate,
                 c -= period
                 if bounds_data is not None:
                     b -= period
-        #--- End: if 
+        # --- End: if
 
         c._custom['direction'] = direction
 
-        if inplace:
-            c = None
         return c
 
-
     # ----------------------------------------------------------------
     # Deprecated attributes and methods
     # ----------------------------------------------------------------
@@ -941,8 +941,8 @@ class DimensionCoordinate(abstract.Coordinate,
         '''
         _DEPRECATION_ERROR_ATTRIBUTE(
             self, 'role',
-            "Use attribute 'construct_type' instead") # pragma: no cover
+            "Use attribute 'construct_type' instead"
+        )  # pragma: no cover
 
-        
-#--- End: class
 
+# --- End: class
