@@ -9,7 +9,7 @@ import numpy
 
 import cf
 
-tmpfile  = tempfile.mktemp('.cf-python_test')
+tmpfile  = tempfile.mktemp('.cf_test')
 tmpfiles = [tmpfile]
 def _remove_tmpfiles():
     '''
@@ -20,7 +20,7 @@ def _remove_tmpfiles():
         except OSError:
             pass
 
-        
+
 atexit.register(_remove_tmpfiles)
 
 class ppTest(unittest.TestCase):
@@ -34,7 +34,7 @@ class ppTest(unittest.TestCase):
         text_file = open(self.new_table, 'w')
         text_file.write('1!24!SURFACE TEMPERATURE AFTER TIMESTEP  !Pa!!!NEW_NAME!!')
         text_file.close()
-        
+
         self.chunk_sizes = (17, 34, 300, 100000)[::-1]
         self.original_chunksize = cf.CHUNKSIZE()
         self.test_only = ()
@@ -60,15 +60,15 @@ class ppTest(unittest.TestCase):
     def test_PP_WGDOS_UNPACKING(self):
         f = cf.read(self.ppfilename)[0]
 
-        self.assertTrue(f.min() > 221.71,
+        self.assertTrue(f.minimum() > 221.71,
                         'Bad unpacking of WGDOS packed data')
-        self.assertTrue(f.max() < 310.45,
+        self.assertTrue(f.maximum() < 310.45,
                         'Bad unpacking of WGDOS packed data')
-        
+
         array = f.array
-    
-        for chunksize in self.chunk_sizes:   
-            cf.CHUNKSIZE(chunksize) 
+
+        for chunksize in self.chunk_sizes:
+            cf.CHUNKSIZE(chunksize)
 
             f = cf.read(self.ppfilename)[0]
 
@@ -85,17 +85,13 @@ class ppTest(unittest.TestCase):
                 self.assertTrue((f.array == array).all(),
                                 'Bad unpacking of WGDOS packed data')
 
-
-#                f.dump()
-#                g.dump()
-                
                 self.assertTrue(f.equals(g, verbose=True),
                                 'Bad writing/reading. format='+fmt)
 
         cf.CHUNKSIZE(self.original_chunksize)
 
 
-#--- End: class
+# --- End: class
 
 if __name__ == '__main__':
     print('Run date:', datetime.datetime.utcnow())
